=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-08-18 20:53:05 +0000
+++ bzrlib/errors.py	2006-08-22 16:14:17 +0000
@@ -1110,12 +1110,3 @@
 
     def __init__(self):
         BzrNewError.__init__(self)    
-
-
-class NoInterWorker(BzrNewError):
-    """No interworker compatible with %(source)s and %(target)s was found."""
-
-    def __init__(self, source, target):
-        self.source = source
-        self.target = target
-        BzrNewError.__init__(self)

=== modified file 'bzrlib/inter.py'
--- bzrlib/inter.py	2006-08-18 20:53:05 +0000
+++ bzrlib/inter.py	2006-08-22 16:14:17 +0000
@@ -18,9 +18,6 @@
 """Inter-object utility class."""
 
 
-from bzrlib import errors
-
-
 class InterObject(object):
     """This class represents operations taking place between two objects.
 
@@ -39,8 +36,8 @@
     the needs_read_lock and needs_write_lock decorators.)
     """
 
-    # _optimisers = set()
-    # Each concrete InterObject type should have its own optimisers set.
+    # _optimisers = list()
+    # Each concrete InterObject type should have its own optimisers list.
 
     def __init__(self, source, target):
         """Construct a default InterObject instance. Please use 'get'.
@@ -77,10 +74,10 @@
         If an optimised worker exists it will be used otherwise
         a default Inter worker instance will be created.
         """
-        for provider in list(klass._optimisers) + [klass]:
+        for provider in reversed(klass._optimisers):
             if provider.is_compatible(source, target):
                 return provider(source, target)
-        raise errors.NoInterWorker(source, target)
+        return klass(source, target)
 
     def lock_read(self):
         """Take out a logical read lock.
@@ -101,7 +98,7 @@
     @classmethod
     def register_optimiser(klass, optimiser):
         """Register an InterObject optimiser."""
-        klass._optimisers.add(optimiser)
+        klass._optimisers.append(optimiser)
 
     def unlock(self):
         """Release the locks on source and target."""

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-08-18 20:53:05 +0000
+++ bzrlib/repository.py	2006-08-22 16:14:17 +0000
@@ -1656,11 +1656,54 @@
     InterRepository.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterRepository types."""
 
+    def copy_content(self, revision_id=None, basis=None):
+        raise NotImplementedError(self.copy_content)
+
+    def fetch(self, revision_id=None, pb=None):
+        raise NotImplementedError(self.fetch)
+   
+    @needs_read_lock
+    def missing_revision_ids(self, revision_id=None):
+        """Return the revision ids that source has that target does not.
+        
+        These are returned in topological order.
+
+        :param revision_id: only return revision ids included by this
+                            revision_id.
+        """
+        # generic, possibly worst case, slow code path.
+        target_ids = set(self.target.all_revision_ids())
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            assert source_ids[0] == None
+            source_ids.pop(0)
+        else:
+            source_ids = self.source.all_revision_ids()
+        result_set = set(source_ids).difference(target_ids)
+        # this may look like a no-op: its not. It preserves the ordering
+        # other_ids had while only returning the members from other_ids
+        # that we've decided we need.
+        return [rev_id for rev_id in source_ids if rev_id in result_set]
+
+
+class InterSameDataRepository(InterRepository):
+    """Code for converting between repositories that represent the same data.
+    
+    Data format and model must match for this to work.
+    """
+
+    _matching_repo_format = RepositoryFormat4()
+    """Repository format for testing with."""
+
     @staticmethod
     def is_compatible(source, target):
+        if not isinstance(source, Repository):
+            return False
+        if not isinstance(target, Repository):
+            return False
         if source._format.rich_root_data == target._format.rich_root_data:
             return True
         else:
@@ -1706,38 +1749,16 @@
         """
         from bzrlib.fetch import GenericRepoFetcher
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
-               self.source, self.source._format, self.target, self.target._format)
+               self.source, self.source._format, self.target, 
+               self.target._format)
         f = GenericRepoFetcher(to_repository=self.target,
                                from_repository=self.source,
                                last_revision=revision_id,
                                pb=pb)
         return f.count_copied, f.failed_revisions
 
-    @needs_read_lock
-    def missing_revision_ids(self, revision_id=None):
-        """Return the revision ids that source has that target does not.
-        
-        These are returned in topological order.
-
-        :param revision_id: only return revision ids included by this
-                            revision_id.
-        """
-        # generic, possibly worst case, slow code path.
-        target_ids = set(self.target.all_revision_ids())
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            assert source_ids[0] == None
-            source_ids.pop(0)
-        else:
-            source_ids = self.source.all_revision_ids()
-        result_set = set(source_ids).difference(target_ids)
-        # this may look like a no-op: its not. It preserves the ordering
-        # other_ids had while only returning the members from other_ids
-        # that we've decided we need.
-        return [rev_id for rev_id in source_ids if rev_id in result_set]
-
-
-class InterWeaveRepo(InterRepository):
+
+class InterWeaveRepo(InterSameDataRepository):
     """Optimised code paths between Weave based repositories."""
 
     _matching_repo_format = RepositoryFormat7()
@@ -1855,7 +1876,7 @@
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
 
-class InterKnitRepo(InterRepository):
+class InterKnitRepo(InterSameDataRepository):
     """Optimised code paths between Knit based repositories."""
 
     _matching_repo_format = RepositoryFormatKnit1()
@@ -1917,6 +1938,7 @@
             # that against the revision records.
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
+InterRepository.register_optimiser(InterSameDataRepository)
 InterRepository.register_optimiser(InterWeaveRepo)
 InterRepository.register_optimiser(InterKnitRepo)
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-08-18 20:53:05 +0000
+++ bzrlib/tests/test_repository.py	2006-08-22 16:14:17 +0000
@@ -370,7 +370,7 @@
             # we should get the default for something InterString returns False
             # to
             self.assertFalse(InterString.is_compatible(dummy_a, None))
-            # self.assertGetsDefaultInterRepository(dummy_a, None)
+            self.assertGetsDefaultInterRepository(dummy_a, None)
             # and we should get an InterString for a pair it 'likes'
             self.assertTrue(InterString.is_compatible(dummy_a, dummy_b))
             inter_repo = repository.InterRepository.get(dummy_a, dummy_b)

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/test_tree.py	2006-08-22 16:14:17 +0000
@@ -79,7 +79,7 @@
         """This test tests the way Tree.compare() uses InterTree."""
         old_optimisers = InterTree._optimisers
         try:
-            InterTree._optimisers = set()
+            InterTree._optimisers = []
             RecordingOptimiser.calls = []
             InterTree.register_optimiser(RecordingOptimiser)
             tree = self.make_branch_and_tree('1')

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-08-18 20:53:05 +0000
+++ bzrlib/tree.py	2006-08-22 16:14:17 +0000
@@ -375,7 +375,7 @@
     will pass through to InterTree as appropriate.
     """
 
-    _optimisers = set()
+    _optimisers = []
 
     @needs_read_lock
     def compare(self, want_unchanged=False, specific_files=None,
@@ -408,7 +408,3 @@
             return delta.TreeDelta()
         return delta._compare_trees(self.source, self.target, want_unchanged,
             specific_file_ids)
-
-    @staticmethod
-    def is_compatible(source, target):
-        return True

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-08-18 20:53:05 +0000
+++ bzrlib/versionedfile.py	2006-08-22 16:14:17 +0000
@@ -558,13 +558,9 @@
     InterVersionedFile.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterVersionedFile types."""
 
-    @staticmethod
-    def is_compatible(source, target):
-        return True
-    
     def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
         """Integrate versions from self.source into self.target.
 

